--
-- mkhref.hs
--
-- Copyright (c) 2003 Minero Aoki <aamine@loveruby.net>
--
-- This program is free software.
-- You can distribute/modify this program under the terms of
-- the GNU LGPL, Lesser General Public License version 2.
-- For details of the GNU LPGL, see the file "COPYING".
--

module Main
  (main)
where

import System
import System.IO
import System.Directory
import Control.Monad
import Data.Char

import HrefUtils

main :: IO ()
main = do prefix <- myGetEnv hrefDataDirEnv
          ok <- doesDirectoryExist prefix
          when (not ok)
              (createDirectory prefix)
          keyfile <- return (prefix ++ "/" ++ hrefKeyListFile)
          ok <- doesFileExist keyfile
          when (ok)
              (removeFile keyfile)
          args <- getArgs
          mapM_ (\a -> processSrc keyfile a $
                         prefix ++ "/" ++ encodePath (basename a)) args

processSrc :: String -> String -> String -> IO ()
processSrc keyfile srcfile destdir = do
    ok <- doesDirectoryExist destdir
    when (not ok)
        (createDirectory destdir)
    cs <- readFile srcfile
    (\chunks -> do putKeyList keyfile (basename srcfile) chunks
                   putChunks destdir chunks) (parse $ lines cs)

putKeyList :: String -> String -> [(String,String)] -> IO ()
putKeyList keyfile modname chunks = do
    f <- openFile keyfile AppendMode
    mapM_ (hPutStrLn f) $ map (\(n,b) -> modname++","++n) chunks
    hClose f

putChunks :: String -> [(String,String)] -> IO ()
putChunks prefix chunks = mapM_ (\(n,c) -> putChunk prefix n c) chunks

putChunk :: String -> String -> String -> IO ()
putChunk prefix func chunk = do
    f <- openFile (prefix ++ "/" ++ encodePath func) WriteMode
    hPutStr f chunk
    hClose f

parse :: [String] -> [(String,String)]
parse [] = []
parse xs = let (header, rest') = break (isSpace.headChar) xs
               (body, rest) = break (not.isSpace.headChar) rest'
           in (getFunctionName header, unlines $ header++body) : parse rest

getFunctionName :: [String] -> String
getFunctionName (x:xs) = let (w:ws) = words x
                         in case head w of
                              '(' -> (tail.init) w
                              _   -> w

basename :: String -> String
basename "/"  = "/"
basename path = last $ split '/' path
